/*
  DS3231 Clock with rotary control modified 14 Feb 2016
  by DongWu

  Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
  library works with all LCD displays that are compatible with the
  Hitachi HD44780 driver. There are many of them out there, and you
  can usually tell them by the 16-pin interface.

   The circuit:
  * LCD RS pin to digital pin 12
  * LCD Enable pin to digital pin 11
  * LCD D4 pin to digital pin 10
  * LCD D5 pin to digital pin 9
  * LCD D6 pin to digital pin 8
  * LCD D7 pin to digital pin 7
  * LCD R/W pin to ground
  * LCD VSS pin to ground
  * LCD VCC pin to 5V
  * 10K resistor:
  * ends to +5V and ground
  * wiper to LCD VO pin (pin 3)

  * DS3231 SCL pin to analog pin 5
  * DS3231 SDA pin to analog pin 4
  * DS3231 VCC pin to 5V
  * DS3231 GND pin to ground

  * Rotary A pin to digital pin 2
  * Rotary B pin to digital pin 3
  * Rotary common pin to ground
  * Rotary pushSwitch pin 1 to digital pin 4
  * Rotary pushSwitch pin 2 to ground

  Library originally added 18 Apr 2008
  by David A. Mellis
  library modified 5 Jul 2009
  by Limor Fried (http://www.ladyada.net)
  example added 9 Jul 2009
  by Tom Igoe
  modified 22 Nov 2010
  by Tom Igoe

  This example code is in the public domain.

  http://www.arduino.cc/en/Tutorial/LiquidCrystal
*/

// include the library code:
#include <LiquidCrystal.h>
#include <Wire.h>
#include "DS3231.h"

// usually the rotary encoders three pins have the ground pin in the middle
enum PinAssignments {
  encoderPinA = 2,   // rigth
  encoderPinB = 3,   // left
  pushButton = 4    // another two pins
};

volatile unsigned int encoderPos = 0;  // a counter for the dial
unsigned int lastReportedPos = 1;   // change management
static boolean rotating = false;      // debounce management

// interrupt service routine vars
boolean A_set = false;              
boolean B_set = false;

boolean buttonReleased = true;

unsigned long previousMillis = 0;        // will store last time LED was updated
unsigned int previousSeconds = 0;        // will store last time temperature was updated

// constants won't change :
 const long interval = 100;           // interval at which to blink (milliseconds)

int temperature;
static const int TemperatureOffset = -20;  //温度偏差修正值（数值为实际偏差值的10倍）
DateTime now;

struct structDateTime{
  int16_t year;
  int8_t month;
  int8_t date;
  int8_t hour;
  int8_t min;
  int8_t sec;
  int8_t wday;
}dtAdj;

int commandConfirm = 0;

static const char weekDay[][4] = {"", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun" };
DateTime dt(2016, 01, 22, 20, 59, 0, 5);  //需设置成当前的日期时间, 格式是 "年, 月, 日, 时, 分, 秒, 星期"

DS3231 RTC; //Create the DS3231 object

static const int dataAdjustItems = 8;
int dataAdjustIndex = -1;
//data position: column, row, digits
unsigned int dataAdjust[][3] = {{3,0,4},{6,0,2},{9,0,2},{15,0,3},{1,1,2},{4,1,2},{7,1,2},{9,1,1}};
//minimum and maximum value of date time
int valueLimit[][2] = {{2000,2199},{1,12},{1,31},{1,7},{0,23},{0,59},{0,59},{1,3}};
static const int daysInMonth[]  = { 31,28,31,30,31,30,31,31,30,31,30,31 };
static const int daysInMonthLeap[]  = { 31,29,31,30,31,30,31,31,30,31,30,31 };
static const char action[][2] {" ", "N", "Y", "X"};

// initialize the library with the numbers of the interface pins
 LiquidCrystal lcd(12, 11, 10, 9, 8, 7);

// make some custom characters: 1602液晶自定义符号
//年
byte Nian[8] = {0b01000,  0b01111,0b10010, 0b01111, 0b01010, 0b11111, 0b00010, 0b00010};
//月
byte Yue[8] = {0b01111, 0b01001, 0b01111, 0b01001, 0b01111, 0b01001, 0b10011, 0b00001};
//日
byte Ri[8] = {0b01111, 0b01001, 0b01001, 0b01111, 0b01001, 0b01001, 0b01111, 0b00000};
//温度符号—摄氏度
byte Du[8] = {0b11000, 0b11000, 0b00111, 0b01000, 0b01000, 0b01000, 0b00111, 0b00000};

void doEncoderA();
void doEncoderB();
void getDateTime();
void displayDateTime(DateTime dt1, int temperatureDisplay);
void displayCursor(int Index);
void changeValue(int index, int delta);
void buttonAction();
int checkDateLimit(int year, int month, int date);
int checkLimit(int Index, int value);
boolean checkLeapYear(int year);

void setup() {
    pinMode(encoderPinA, INPUT); 
    pinMode(encoderPinB, INPUT); 
    pinMode(pushButton, INPUT);
    // turn on pullup resistors
    digitalWrite(encoderPinA, HIGH);
    digitalWrite(encoderPinB, HIGH);
    digitalWrite(pushButton, HIGH);
  
    //Serial.begin(57600);
    Wire.begin();
    RTC.begin();
    //Adjust date-time as defined 'dt' above
    //下面一行语句将时钟设置成当前的时间和日期. 需要设定时间和日期时, 将行前的注释符号"//"去掉
    //RTC.adjust(dt); 
   
    // set up the LCD's number of columns and rows:
    lcd.begin(16, 2);
     // create a new character
    lcd.createChar(0, Nian);
    lcd.createChar(1, Yue);
    lcd.createChar(2, Ri);
    lcd.createChar(3, Du);
    lcd.clear();
}

void loop() {
  rotating = true;  // reset the debouncer

  if (lastReportedPos != encoderPos) {
    //Serial.print("Index:");
    //Serial.println(encoderPos, DEC);
    lastReportedPos = encoderPos;
  }
  if ((digitalRead(pushButton) == LOW) && (buttonReleased == true))  {
    delay(5);
    //button pressed
    if (digitalRead(pushButton) == LOW){
      encoderPos = 0;
      dataAdjustIndex += 1;
      if (dataAdjustIndex >= dataAdjustItems){
        dataAdjustIndex = -1;
      }
      if (commandConfirm == 1 && dataAdjustIndex == -1)
      dataAdjustIndex += 1;
      
      buttonReleased = false;
      //Serial.print("dataAdjustIndex:");
      //Serial.println(dataAdjustIndex);
      commandConfirm = buttonAction(commandConfirm);
      
      if (dataAdjustIndex > -1){
        commandConfirm = 1;
        dt = DateTime(dtAdj.year,dtAdj.month,dtAdj.date,dtAdj.hour,dtAdj.min,dtAdj.sec,dtAdj.wday);
        displayDateTime(dt, temperature, (char*)action[commandConfirm]);
        displayCursor(dataAdjustIndex);
        // encoder pin on interrupt 0 (pin 2)
        attachInterrupt(0, doEncoderA, CHANGE);
        // encoder pin on interrupt 1 (pin 3)
        attachInterrupt(1, doEncoderB, CHANGE);
      }
      else{
        detachInterrupt(0);
        detachInterrupt(1);
      }      
    }
  }
  if (digitalRead(pushButton) == HIGH){
  buttonReleased = true;
  }
  
  if (dataAdjustIndex == -1){
    getDateTime();
    displayDateTime(now, temperature, (char*)action[commandConfirm]);    
  }
}

void getDateTime(){
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
  
      now = RTC.now(); //get the current date-time
      dtAdj.year = now.year();
      dtAdj.month = now.month();
      dtAdj.date = now.date();
      dtAdj.wday = now.dayOfWeek();
      dtAdj.hour = now.hour();
      dtAdj.min = now.minute();
      dtAdj.sec = now.second();
      //read temperature data once a second
      if (now.second() != previousSeconds){
        previousSeconds = now.second();   
        RTC.convertTemperature();  //get temperature
        temperature = int(RTC.getTemperature()*10) + TemperatureOffset;
      }    
    }
}
void displayDateTime(DateTime dt1, int temperatureDisplay, char act[2]){    
    int temperature100 = temperatureDisplay/100;
    int temperature10=(temperatureDisplay%100)/10;
    int temperature1=(temperatureDisplay%100)%10; 
    
    //display date
    lcd.setCursor(0, 0);
    lcd.print(dt1.year(),DEC);
    lcd.setCursor(4, 0);
    lcd.write(byte(0));
    lcd.print(dt1.month()/10,DEC);
    lcd.print(dt1.month()%10,DEC);
    lcd.write((byte)1);
    lcd.print(dt1.date()/10,DEC);
    lcd.print(dt1.date()%10,DEC);
    lcd.write((byte)2);
    lcd.setCursor(13, 0);
    lcd.write(weekDay[dt1.dayOfWeek()]);

    //display time
    lcd.setCursor(0, 1);
    lcd.print(dt1.hour()/10,DEC);
    lcd.print(dt1.hour()%10,DEC);
    lcd.print(':');
    lcd.print(dt1.minute()/10,DEC);
    lcd.print(dt1.minute()%10,DEC);
    lcd.print(':');
    lcd.print(dt1.second()/10,DEC);
    lcd.print(dt1.second()%10,DEC);

    //display button command
    lcd.setCursor(9, 1);
    lcd.print(act);
    
    //display temperature
    lcd.setCursor(11, 1); 
    lcd.print(temperature100,DEC);
    lcd.print(temperature10,DEC);
    lcd.print('.');
    lcd.print(temperature1,DEC);
    lcd.write((byte)3);   
} 

// Interrupt on A changing state
void doEncoderA(){
  // debounce
  if ( rotating ) delay (2);  // wait a little until the bouncing is done

  // Test transition, did things really change? 
  if( digitalRead(encoderPinA) != A_set ) {  // debounce once more
    A_set = !A_set;

    // adjust counter + if A leads B
    if ( A_set && !B_set ) {
      encoderPos += 1;
      changeValue(dataAdjustIndex, 1);
      dt = DateTime(dtAdj.year,dtAdj.month,dtAdj.date,dtAdj.hour,dtAdj.min,dtAdj.sec,dtAdj.wday);
      displayDateTime(dt, temperature, (char*)action[commandConfirm]);
      displayCursor(dataAdjustIndex);
      //lcd.setCursor(dataAdjust[dataAdjustIndex][0], dataAdjust[dataAdjustIndex][1]);
    }
    rotating = false;  // no more debouncing until loop() hits again
  }
}

// Interrupt on B changing state, same as A above
void doEncoderB(){
  if ( rotating ) delay (2);
  if( digitalRead(encoderPinB) != B_set ) {
    B_set = !B_set;
    //  adjust counter - 1 if B leads A
    if( B_set && !A_set ) {
      encoderPos -= 1;
      changeValue(dataAdjustIndex, -1);
      dt = DateTime(dtAdj.year,dtAdj.month,dtAdj.date,dtAdj.hour,dtAdj.min,dtAdj.sec,dtAdj.wday);
      displayDateTime(dt, temperature, (char*)action[commandConfirm]);
      displayCursor(dataAdjustIndex);
      //lcd.setCursor(dataAdjust[dataAdjustIndex][0], dataAdjust[dataAdjustIndex][1]);
    }
    rotating = false;
  }
}

void displayCursor(int Index){  
  lcd.setCursor(dataAdjust[Index][0], dataAdjust[Index][1]);
  lcd.cursor();
  //lcd.blink();  
}

void changeValue(int Index, int delta){
  switch(Index){
    case 0:
      dtAdj.year += delta;
      dtAdj.year = checkLimit(Index, dtAdj.year);
      checkDateLimit(dtAdj.year, dtAdj.month);
      changeValue(2, 0);
      break;
    case 1:
      dtAdj.month += delta;
      dtAdj.month = checkLimit(Index, dtAdj.month);
      checkDateLimit(dtAdj.year, dtAdj.month);
      changeValue(2, 0);
      break;
    case 2:
      dtAdj.date += delta;
      dtAdj.date = checkLimit(Index, dtAdj.date);
      break;
    case 3:
      dtAdj.wday += delta;
      dtAdj.wday = checkLimit(Index, dtAdj.wday);
      break;
    case 4:
      dtAdj.hour += delta;
      dtAdj.hour = checkLimit(Index, dtAdj.hour);
      break;
    case 5:
      dtAdj.min += delta;
      dtAdj.min = checkLimit(Index, dtAdj.min);
      break;
    case 6:
      dtAdj.sec += delta;
      dtAdj.sec = checkLimit(Index, dtAdj.sec);
      break;
    case 7:
      commandConfirm += delta;
      commandConfirm = checkLimit(Index, commandConfirm);
      break;
    case 8:
      break;
  }
}

int buttonAction(int act){
    switch(act){
      case 0:
        return 0;
        break;
      case 1:
        return 1;
        break;
      case 2:
        RTC.adjust(DateTime(dtAdj.year,dtAdj.month,dtAdj.date,dtAdj.hour,dtAdj.min,dtAdj.sec,dtAdj.wday));
        return 0;
        break;
      case 3:
        return 0;
        break;      
    }  
}

int checkDateLimit(int year, int month){
  if(checkLeapYear(year)){
    valueLimit[2][1]=daysInMonthLeap[month-1];
  }  
  else{
    valueLimit[2][1]=daysInMonth[month-1];
  }
}

int checkLimit(int index, int value){
  if (value > valueLimit[index][1]){
    return valueLimit[index][0];
  }
  if (value < valueLimit[index][0]){      
    return valueLimit[index][1];
  }
  return value;   
}

boolean checkLeapYear(int year){
  if(year%4 == 0 && year%100 != 0 || year%400 == 0)
  return true;
  else
  return false;
}
